@@ -1,22 +1,17 @@
 require 'mkmf'
 
 case RUBY_PLATFORM
-when /bccwin32/
+when /(ms|bcc)win32|mingw/
   test_func = "WSACleanup"
   have_library("ws2_32", "WSACleanup")
-when /mswin32|mingw/
-  test_func = "WSACleanup"
-  if with_config("winsock2")
-    have_library("ws2_32", "WSACleanup")
-  else
-    have_library("wsock32", "WSACleanup")
-  end
 when /cygwin/
   test_func = "socket"
 when /beos/
   test_func = "socket"
   have_library("net", "socket")
-  have_func("closesocket")
+when /haiku/
+  test_func = "socket"
+  have_library("network", "socket")
 when /i386-os2_emx/
   test_func = "socket"
   have_library("socket", "socket")
@@ -33,13 +28,17 @@ if /solaris/ =~ RUBY_PLATFORM and !try_compile("")
   # bug of gcc 3.0 on Solaris 8 ?
   headers << "sys/feature_tests.h"
 end
+if have_header("arpa/inet.h")
+  headers << "arpa/inet.h"
+end
 
 ipv6 = false
-default_ipv6 = /cygwin/ !~ RUBY_PLATFORM
+default_ipv6 = /cygwin|beos|haiku/ !~ RUBY_PLATFORM
 if enable_config("ipv6", default_ipv6)
   if checking_for("ipv6") {try_link(<<EOF)}
 #include <sys/types.h>
 #include <sys/socket.h>
+int
 main()
 {
   socket(AF_INET6, SOCK_STREAM, 0);
@@ -91,17 +90,10 @@ if have_struct_member("struct sockaddr_in", "sin_len", headers)
 end
 
 #   doug's fix, NOW add -Dss_family... only if required!
-[nil, " -Dss_family=__ss_family -Dss_len=__ss_len"].each do |flags|
-  if flags
-    cppflags = $CPPFLAGS
-    $CPPFLAGS += flags
-  end
-  if have_struct_member("struct sockaddr_storage", "ss_family", headers)
-    $defs[-1] = "-DHAVE_SOCKADDR_STORAGE"
-    break
-  elsif flags
-    $CPPFLAGS = cppflags
-  end
+doug = proc {have_struct_member("struct sockaddr_storage", "ss_family", headers)}
+if (doug[] or
+    with_cppflags($CPPFLAGS + " -Dss_family=__ss_family -Dss_len=__ss_len", &doug))
+  $defs[-1] = "-DHAVE_SOCKADDR_STORAGE"
 end
 
 if have_struct_member("struct sockaddr", "sa_len", headers)
@@ -132,11 +124,17 @@ getaddr_info_ok = enable_config("wide-getaddrinfo") do
 #define AF_LOCAL AF_UNIX
 #endif
 
+int
 main()
 {
   int passive, gaierr, inet4 = 0, inet6 = 0;
   struct addrinfo hints, *ai, *aitop;
   char straddr[INET6_ADDRSTRLEN], strport[16];
+#ifdef _WIN32
+  WSADATA retdata;
+
+  WSAStartup(MAKEWORD(2, 0), &retdata);
+#endif
 
   for (passive = 0; passive <= 1; passive++) {
     memset(&hints, 0, sizeof(hints));
@@ -153,6 +151,10 @@ main()
       if (ai->ai_addr == NULL)
         goto bad;
 #if defined(_AIX)
+      if (ai->ai_family == AF_INET6 && passive) {
+        inet6++;
+        continue;
+      }
       ai->ai_addr->sa_len = ai->ai_addrlen;
       ai->ai_addr->sa_family = ai->ai_family;
 #endif
@@ -242,7 +244,7 @@ end
 
 $objs = ["socket.#{$OBJEXT}"]
 
-unless getaddr_info_ok and have_func("getnameinfo", "netdb.h") and have_func("getaddrinfo", "netdb.h")
+unless getaddr_info_ok and have_func("getnameinfo", headers) and have_func("getaddrinfo", headers)
   if have_struct_member("struct in6_addr", "s6_addr8", headers)
     $defs[-1] = "-DHAVE_ADDR8"
   end
@@ -252,7 +254,19 @@ unless getaddr_info_ok and have_func("getnameinfo", "netdb.h") and have_func("ge
   have_func("inet_ntop") or have_func("inet_ntoa")
   have_func("inet_pton") or have_func("inet_aton")
   have_func("getservbyport")
-  have_header("arpa/inet.h")
+  if have_func("gai_strerror")
+    unless checking_for("gai_strerror() returns const pointer") {!try_compile(<<EOF)}
+#{cpp_include(headers)}
+#include <stdlib.h>
+void
+conftest_gai_strerror_is_const()
+{
+    *gai_strerror(0) = 0;
+}
+EOF
+      $defs << "-DGAI_STRERROR_CONST"
+    end
+  end
   have_header("arpa/nameser.h")
   have_header("resolv.h")
 end
@@ -264,6 +278,8 @@ end
 have_header("sys/un.h")
 have_header("sys/uio.h")
 
+$distcleanfiles << "constants.h"
+
 if have_func(test_func)
   have_func("hsterror")
   have_func("getipnodebyname") or have_func("gethostbyname2")
