@@ -2,7 +2,7 @@
 
   marshal.c -
 
-  $Author: nobu $
+  $Author: yugui $
   created at: Thu Apr 27 16:30:01 JST 1995
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -79,7 +79,7 @@ shortlen(long len, BDIGIT *ds)
 #define TYPE_LINK	'@'
 
 static ID s_dump, s_load, s_mdump, s_mload;
-static ID s_dump_data, s_load_data, s_alloc;
+static ID s_dump_data, s_load_data, s_alloc, s_call;
 static ID s_getbyte, s_read, s_write, s_binmode;
 
 ID rb_id_encoding(void);
@@ -150,10 +150,11 @@ struct dump_call_arg {
 };
 
 static void
-check_dump_arg(struct dump_arg *arg)
+check_dump_arg(struct dump_arg *arg, ID sym)
 {
     if (!DATA_PTR(arg->wrapper)) {
-	rb_raise(rb_eRuntimeError, "Marshal.dump reentered");
+        rb_raise(rb_eRuntimeError, "Marshal.dump reentered at %s",
+		 rb_id2name(sym));
     }
 }
 
@@ -495,7 +496,7 @@ w_encoding(VALUE obj, long num, struct dump_call_arg *arg)
 	name = (st_data_t)rb_str_new2(rb_enc_name(enc));
 	st_insert(arg->arg->encodings, (st_data_t)rb_enc_name(enc), name);
     } while (0);
-    w_object(name, arg->arg, arg->limit);
+    w_object(name, arg->arg, arg->limit + 1);
 }
 
 static void
@@ -591,7 +592,7 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
             st_add_direct(arg->data, obj, arg->data->num_entries);
 
 	    v = rb_funcall(obj, s_mdump, 0, 0);
-	    check_dump_arg(arg);
+	    check_dump_arg(arg, s_mdump);
 	    w_class(TYPE_USRMARSHAL, obj, arg, Qfalse);
 	    w_object(v, arg, limit);
 	    if (hasiv) w_ivar(obj, 0, &c_arg);
@@ -603,7 +604,7 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
             int hasiv2;
 
 	    v = rb_funcall(obj, s_dump, 1, INT2NUM(limit));
-	    check_dump_arg(arg);
+	    check_dump_arg(arg, s_dump);
 	    if (TYPE(v) != T_STRING) {
 		rb_raise(rb_eTypeError, "_dump() must return string");
 	    }
@@ -771,7 +772,7 @@ w_object(VALUE obj, struct dump_arg *arg, int limit)
 			     rb_obj_classname(obj));
 		}
 		v = rb_funcall(obj, s_dump_data, 0);
-		check_dump_arg(arg);
+		check_dump_arg(arg, s_dump_data);
 		w_class(TYPE_DATA, obj, arg, Qtrue);
 		w_object(v, arg, limit);
 	    }
@@ -865,29 +866,30 @@ marshal_dump(int argc, VALUE *argv)
 	else port = a1;
     }
     arg.dest = 0;
-    if (!NIL_P(port)) {
-	if (!rb_respond_to(port, s_write)) {
-	  type_error:
-	    rb_raise(rb_eTypeError, "instance of IO needed");
-	}
-	arg.str = rb_str_buf_new(0);
-	arg.dest = port;
-	if (rb_respond_to(port, s_binmode)) {
-	    rb_funcall2(port, s_binmode, 0, 0);
-	}
-    }
-    else {
-	port = rb_str_buf_new(0);
-	arg.str = port;
-    }
-
     arg.symbols = st_init_numtable();
     arg.data    = st_init_numtable();
     arg.taint   = Qfalse;
     arg.untrust = Qfalse;
     arg.compat_tbl = st_init_numtable();
-    arg.wrapper = Data_Wrap_Struct(rb_cData, mark_dump_arg, 0, &arg);
     arg.encodings = 0;
+    arg.str = rb_str_buf_new(0);
+    RBASIC(arg.str)->klass = 0;
+    arg.wrapper = Data_Wrap_Struct(rb_cData, mark_dump_arg, 0, &arg);
+    if (!NIL_P(port)) {
+	if (!rb_respond_to(port, s_write)) {
+	  type_error:
+	    rb_raise(rb_eTypeError, "instance of IO needed");
+	}
+	arg.dest = port;
+	if (rb_respond_to(port, s_binmode)) {
+	    rb_funcall2(port, s_binmode, 0, 0);
+	    check_dump_arg(&arg, s_binmode);
+	}
+    }
+    else {
+	port = arg.str;
+    }
+
     c_arg.obj   = obj;
     c_arg.arg   = &arg;
     c_arg.limit = limit;
@@ -896,6 +898,7 @@ marshal_dump(int argc, VALUE *argv)
     w_byte(MARSHAL_MINOR, &arg);
 
     rb_ensure(dump, (VALUE)&c_arg, dump_ensure, (VALUE)&arg);
+    RBASIC(arg.str)->klass = rb_cString;
 
     return port;
 }
@@ -904,22 +907,33 @@ struct load_arg {
     VALUE src;
     long offset;
     st_table *symbols;
-    VALUE data;
+    st_table *data;
     VALUE proc;
     int taint;
     int untrust;
     st_table *compat_tbl;
-    VALUE compat_tbl_wrapper;
+    VALUE wrapper;
 };
 
 static void
-check_load_arg(struct load_arg *arg)
+check_load_arg(struct load_arg *arg, ID sym)
 {
-    if (!DATA_PTR(arg->compat_tbl_wrapper)) {
-	rb_raise(rb_eRuntimeError, "Marshal.load reentered");
+    if (!DATA_PTR(arg->wrapper)) {
+        rb_raise(rb_eRuntimeError, "Marshal.load reentered at %s",
+		 rb_id2name(sym));
     }
 }
 
+static void
+mark_load_arg(void *ptr)
+{
+    struct load_arg *p = ptr;
+    if (!ptr)
+        return;
+    rb_mark_tbl(p->data);
+    rb_mark_hash(p->compat_tbl);
+}
+
 static VALUE r_entry(VALUE v, struct load_arg *arg);
 static VALUE r_object(struct load_arg *arg);
 static VALUE path2class(const char *path);
@@ -940,7 +954,7 @@ r_byte(struct load_arg *arg)
     else {
 	VALUE src = arg->src;
 	VALUE v = rb_funcall2(src, s_getbyte, 0, 0);
-	check_load_arg(arg);
+	check_load_arg(arg, s_getbyte);
 	if (NIL_P(v)) rb_eof_error();
 	c = (unsigned char)NUM2CHR(v);
     }
@@ -1017,7 +1031,7 @@ r_bytes0(long len, struct load_arg *arg)
 	VALUE src = arg->src;
 	VALUE n = LONG2NUM(len);
 	str = rb_funcall2(src, s_read, 1, &n);
-	check_load_arg(arg);
+	check_load_arg(arg, s_read);
 	if (NIL_P(str)) goto too_short;
 	StringValue(str);
 	if (RSTRING_LEN(str) != len) goto too_short;
@@ -1083,10 +1097,10 @@ r_entry(VALUE v, struct load_arg *arg)
 {
     st_data_t real_obj = (VALUE)Qundef;
     if (st_lookup(arg->compat_tbl, v, &real_obj)) {
-        rb_hash_aset(arg->data, INT2FIX(RHASH_SIZE(arg->data)), (VALUE)real_obj);
+        st_insert(arg->data, arg->data->num_entries, (st_data_t)real_obj);
     }
     else {
-        rb_hash_aset(arg->data, INT2FIX(RHASH_SIZE(arg->data)), v);
+        st_insert(arg->data, arg->data->num_entries, (st_data_t)v);
     }
     if (arg->taint) {
         OBJ_TAINT(v);
@@ -1117,8 +1131,8 @@ r_leave(VALUE v, struct load_arg *arg)
         v = real_obj;
     }
     if (arg->proc) {
-	v = rb_funcall(arg->proc, rb_intern("call"), 1, v);
-	check_load_arg(arg);
+	v = rb_funcall(arg->proc, s_call, 1, v);
+	check_load_arg(arg, s_call);
     }
     return v;
 }
@@ -1193,18 +1207,18 @@ r_object0(struct load_arg *arg, int *ivp, VALUE extmod)
     VALUE v = Qnil;
     int type = r_byte(arg);
     long id;
+    st_data_t link;
 
     switch (type) {
       case TYPE_LINK:
 	id = r_long(arg);
-	v = rb_hash_aref(arg->data, LONG2FIX(id));
-	check_load_arg(arg);
-	if (NIL_P(v)) {
+	if (!st_lookup(arg->data, (st_data_t)id, &link)) {
 	    rb_raise(rb_eArgError, "dump format error (unlinked)");
 	}
+	v = (VALUE)link;
 	if (arg->proc) {
-	    v = rb_funcall(arg->proc, rb_intern("call"), 1, v);
-	    check_load_arg(arg);
+	    v = rb_funcall(arg->proc, s_call, 1, v);
+	    check_load_arg(arg, s_call);
 	}
 	break;
 
@@ -1296,7 +1310,7 @@ r_object0(struct load_arg *arg, int *ivp, VALUE extmod)
 		d = strtod(ptr, &e);
 		d = load_mantissa(d, e, RSTRING_LEN(str) - (e - ptr));
 	    }
-	    v = DOUBLE2NUM(d);
+	    v = DBL2NUM(d);
 	    v = r_entry(v, arg);
             v = r_leave(v, arg);
 	}
@@ -1357,8 +1371,20 @@ r_object0(struct load_arg *arg, int *ivp, VALUE extmod)
 	{
 	    volatile VALUE str = r_bytes(arg);
 	    int options = r_byte(arg);
-	    v = r_entry(rb_reg_new_str(str, options), arg);
-            v = r_leave(v, arg);
+	    v = rb_reg_new("", 0, options);
+	    if (ivp) {
+		r_ivar(v, arg);
+		*ivp = Qfalse;
+	    }
+	    if (rb_enc_get_index(v) != rb_usascii_encindex())
+		rb_enc_copy(str, v);
+	    if (rb_enc_get_index(str) != rb_utf8_encindex()) {
+#define f_gsub_bang(x,y,z) rb_funcall(x, rb_intern("gsub!"), 2, y, z)
+		f_gsub_bang(str, rb_reg_new("\\\\u", 3, 0), rb_usascii_str_new_cstr("u"));
+	    }
+	    str = r_entry(rb_reg_new_str(str, options), arg);
+	    rb_copy_generic_ivar(str, v);
+	    v = r_leave(str, arg);
 	}
 	break;
 
@@ -1448,7 +1474,7 @@ r_object0(struct load_arg *arg, int *ivp, VALUE extmod)
 		*ivp = Qfalse;
 	    }
 	    v = rb_funcall(klass, s_load, 1, data);
-	    check_load_arg(arg);
+	    check_load_arg(arg, s_load);
 	    v = r_entry(v, arg);
             v = r_leave(v, arg);
 	}
@@ -1473,7 +1499,7 @@ r_object0(struct load_arg *arg, int *ivp, VALUE extmod)
 	    v = r_entry(v, arg);
 	    data = r_object(arg);
 	    rb_funcall(v, s_mload, 1, data);
-	    check_load_arg(arg);
+	    check_load_arg(arg, s_mload);
             v = r_leave(v, arg);
 	}
         break;
@@ -1500,7 +1526,7 @@ r_object0(struct load_arg *arg, int *ivp, VALUE extmod)
 		   warn = Qfalse;
 	       }
 	       v = rb_funcall(klass, s_alloc, 0);
-	       check_load_arg(arg);
+	       check_load_arg(arg, s_alloc);
            }
 	   else {
 	       v = rb_obj_alloc(klass);
@@ -1515,7 +1541,7 @@ r_object0(struct load_arg *arg, int *ivp, VALUE extmod)
                         rb_class2name(klass));
            }
            rb_funcall(v, s_load_data, 1, r_object0(arg, 0, extmod));
-	   check_load_arg(arg);
+	   check_load_arg(arg, s_load_data);
            v = r_leave(v, arg);
        }
        break;
@@ -1581,11 +1607,12 @@ load(struct load_arg *arg)
 static VALUE
 load_ensure(struct load_arg *arg)
 {
-    if (!DATA_PTR(arg->compat_tbl_wrapper)) return 0;
+    if (!DATA_PTR(arg->wrapper)) return 0;
     st_free_table(arg->symbols);
+    st_free_table(arg->data);
     st_free_table(arg->compat_tbl);
-    DATA_PTR(arg->compat_tbl_wrapper) = 0;
-    arg->compat_tbl_wrapper = 0;
+    DATA_PTR(arg->wrapper) = 0;
+    arg->wrapper = 0;
     return 0;
 }
 
@@ -1626,8 +1653,11 @@ marshal_load(int argc, VALUE *argv)
     arg.untrust = OBJ_UNTRUSTED(port);
     arg.src = port;
     arg.offset = 0;
+    arg.symbols = st_init_numtable();
+    arg.data    = st_init_numtable();
     arg.compat_tbl = st_init_numtable();
-    arg.compat_tbl_wrapper = Data_Wrap_Struct(rb_cData, rb_mark_tbl, 0, arg.compat_tbl);
+    arg.proc = 0;
+    arg.wrapper = Data_Wrap_Struct(rb_cData, mark_load_arg, 0, &arg);
 
     major = r_byte(&arg);
     minor = r_byte(&arg);
@@ -1642,11 +1672,7 @@ marshal_load(int argc, VALUE *argv)
 		MARSHAL_MAJOR, MARSHAL_MINOR, major, minor);
     }
 
-    arg.symbols = st_init_numtable();
-    arg.data   = rb_hash_new();
-    RBASIC(arg.data)->klass = 0;
-    if (NIL_P(proc)) arg.proc = 0;
-    else             arg.proc = proc;
+    if (!NIL_P(proc)) arg.proc = proc;
     v = rb_ensure(load, (VALUE)&arg, load_ensure, (VALUE)&arg);
 
     return v;
@@ -1700,6 +1726,7 @@ Init_marshal(void)
     s_dump_data = rb_intern("_dump_data");
     s_load_data = rb_intern("_load_data");
     s_alloc = rb_intern("_alloc");
+    s_call = rb_intern("call");
     s_getbyte = rb_intern("getbyte");
     s_read = rb_intern("read");
     s_write = rb_intern("write");
@@ -1713,9 +1740,9 @@ Init_marshal(void)
     rb_define_const(rb_mMarshal, "MINOR_VERSION", INT2FIX(MARSHAL_MINOR));
 
     compat_allocator_tbl = st_init_numtable();
-    rb_gc_register_address(&compat_allocator_tbl_wrapper);
     compat_allocator_tbl_wrapper =
 	Data_Wrap_Struct(rb_cData, mark_marshal_compat_t, 0, compat_allocator_tbl);
+    rb_gc_register_mark_object(compat_allocator_tbl_wrapper);
 }
 
 VALUE
