@@ -1085,21 +1085,6 @@ class TestIO < Test::Unit::TestCase
     end
   end
 
-  def test_pos
-    t = make_tempfile
-
-    open(t.path, IO::RDWR|IO::CREAT|IO::TRUNC, 0600) do |f|
-      f.write "Hello"
-      assert_equal(5, f.pos)
-    end
-    open(t.path, IO::RDWR|IO::CREAT|IO::TRUNC, 0600) do |f|
-      f.sync = true
-      f.read
-      f.write "Hello"
-      assert_equal(5, f.pos)
-    end
-  end
-
   def test_sysseek
     t = make_tempfile
 
@@ -1383,16 +1368,4 @@ class TestIO < Test::Unit::TestCase
       }
     }
   end
-
-  def test_tainted
-    t = make_tempfile
-    assert(File.read(t.path, 4).tainted?, '[ruby-dev:38826]')
-    assert(File.open(t.path) {|f| f.read(4)}.tainted?, '[ruby-dev:38826]')
-  end
-
-  def test_binmode_after_closed
-    t = make_tempfile
-    t.close
-    assert_raise(IOError) {t.binmode}
-  end
 end
